﻿//-----------------------------------------------------------------------------
// <copyright file="DDSPreferences.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Preferences of a DDS.</summary>
//-----------------------------------------------------------------------------
namespace DDSCommunication
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;

    /// <summary>
    /// Preferences of a DDS.
    /// </summary>
    public class DDSPreferences
    {
        // public DDSPreferences.Modulation ModulationPreferences { get; internal set; }

        /// <summary>
        /// Enum of all available signal forms. The values match the used values of the DDS.
        /// </summary>
        public enum SignalForm
        {
            /// <summary>
            /// Indicates sinus signal form.
            /// </summary>
            Sinus = 0x00,

            /// <summary>
            /// Indicates triangle signal form.
            /// </summary>
            Triangle = 0x01,

            /// <summary>
            /// Indicates rectangle (full frequency) signal form.
            /// </summary>
            RectangleFullFrequency = 0x02,

            /// <summary>
            /// Indicates rectangle (half frequency) signal form.
            /// </summary>
            RectangleHalfFrequency = 0x03
        }

        /// <summary>
        /// Enum for changing the mode of the DDS.
        /// </summary>
        public enum Mode
        {
            /// <summary>
            /// Activate standard mode.
            /// </summary>
            Standard = 0x00,

            /// <summary>
            /// Activate wobbling mode.
            /// </summary>
            Wobbling = 0x01,

            /// <summary>
            /// Activate modulation mode.
            /// </summary>
            Modulation = 0x02
        }

        /// <summary>
        /// Gets the signal form.
        /// </summary>
        /// <value>Signal Form.</value>
        public DDSPreferences.SignalForm DeviceSignalForm { get; internal set; }

        /// <summary>
        /// Gets version of the DDS.
        /// </summary>
        /// <value>Version of the DDS.</value>
        public string Version { get; internal set; }

        /// <summary>
        /// Gets the frequency.
        /// </summary>
        /// <value>Current frequency.</value>
        public double Frequency { get; internal set; }

        /// <summary>
        /// Gets the minimal frequency.
        /// </summary>
        /// <value>Current minimal frequency.</value>
        public double MinimalFrequency { get; internal set; }

        /// <summary>
        /// Gets the maximal frequency.
        /// </summary>
        /// <value>Current maximal frequency.</value>
        public double MaximalFrequency { get; internal set; }

        /// <summary>
        /// Gets the frequency step size.
        /// </summary>
        /// <value>Frequency step size.</value>
        public double FrequencyStepSize { get; internal set; }

        /// <summary>
        /// Gets the amplitude.
        /// </summary>
        /// <value>Amplitude of the DDS.</value>
        public double Amplitude { get; internal set; }

        /// <summary>
        /// Gets the wobble start frequency.
        /// </summary>
        /// <value>Wobble start frequency.</value>
        public double WobbleStartFrequency { get; internal set; }

        /// <summary>
        /// Gets the wobble stop frequency.
        /// </summary>
        /// <value>Wobble stop frequency.</value>
        public double WobbleStopFrequency { get; internal set; }

        /// <summary>
        /// Gets the wobble frequency.
        /// </summary>
        /// <value>Wobble frequency.</value>
        public double WobbleFrequency { get; internal set; }

        /// <summary>
        /// Gets or sets current mode of the DDS.
        /// </summary>
        /// <value>Current mode of the DDS.</value>
        public DDSPreferences.Mode DeviceMode { get; set; }
    }
}