﻿// <auto-generated/>
//-----------------------------------------------------------------------------
// <copyright file="SiUSBXpDevice.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Class that represents the communication with the device</summary>
//-----------------------------------------------------------------------------
namespace DDSCommunication.USB
{
    using System;
    using System.Collections.Generic;
    using System.Text;
    using System.Threading;

    /// <summary>
    /// Class that represents the communication with the device.
    /// </summary>
    public class SiUSBXpDevice
    {
        /// <summary>
        /// ASCII-control-character for a start sign.
        /// </summary>
        public const byte STX = 0x02;

        /// <summary>
        /// ASCII-control-character for an acknowledge.
        /// </summary>
        public const byte ACK = 0x06;

        /// <summary>
        /// ASCII-control-character for communication.
        /// </summary>
        public const byte ENQ = 0x10;

        /// <summary>
        /// Connection state.
        /// </summary>
        private bool isConnected = false;

        /// <summary>
        /// Vendor identification number.
        /// </summary>
        private ushort vid;

        /// <summary>
        /// Product identification number.
        /// </summary>
        private ushort pid;

        /// <summary>
        /// Databits with default value.
        /// </summary>
        private ushort databits = 8;

        /// <summary>
        /// Parity with default value.
        /// </summary>
        private ushort parity = 2;

        /// <summary>
        /// Stopbits with default value.
        /// </summary>
        private ushort stopbit = 0;

        /// <summary>
        /// Baudrate with default value.
        /// </summary>
        private int baudrate = 38400;

        /// <summary>
        /// Readtimeout with default value.
        /// </summary>
        private int rtimeout = 500;

        /// <summary>
        /// Writetimeout with default value.
        /// </summary>
        private int wtimeout = 500;

        /// <summary>
        /// Initializes a new instance of the SiUSBXpDevice class.
        /// </summary>
        /// <param name="vid">Vendor identification number.</param>
        /// <param name="pid">Product identification number.</param>
        public SiUSBXpDevice(ushort vid, ushort pid)
        {
            this.pid = pid;
            this.vid = vid;

            this.Connect();
        }

        /// <summary>
        /// Initializes a new instance of the SiUSBXpDevice class.
        /// </summary>
        /// <param name="vid">Vendor identification number.</param>
        /// <param name="pid">Product identification number.</param>
        /// <param name="databits">Databits setting.</param>
        /// <param name="parity">Parity setting.</param>
        /// <param name="stopbit">Stopbits setting.</param>
        /// <param name="baudrate">Baudrate setting.</param>
        public SiUSBXpDevice(ushort vid, ushort pid, ushort databits, ushort parity, ushort stopbit, int baudrate)
        {
            this.pid = pid;
            this.vid = vid;

            this.databits = databits;
            this.parity = parity;
            this.stopbit = stopbit;

            this.baudrate = baudrate;

            this.Connect();
        }

        /// <summary>
        /// Initializes a new instance of the SiUSBXpDevice class.
        /// </summary>
        /// <param name="vid">Vendor identification number.</param>
        /// <param name="pid">Product identification number.</param>
        /// <param name="databits">Databits setting.</param>
        /// <param name="parity">Parity setting.</param>
        /// <param name="stopbit">Stopbits setting.</param>
        /// <param name="baudrate">Baudrate setting.</param>
        /// <param name="rtimeout">Read timeout.</param>
        /// <param name="wtimeout">Write timeout.</param>
        public SiUSBXpDevice(ushort vid, ushort pid, ushort databits, ushort parity, ushort stopbit, int baudrate, int rtimeout, int wtimeout)
        {
            this.pid = pid;
            this.vid = vid;

            this.databits = databits;
            this.parity = parity;
            this.stopbit = stopbit;

            this.baudrate = baudrate;
            this.rtimeout = rtimeout;
            this.wtimeout = wtimeout;

            this.Connect();
        }

        /// <summary>
        /// Gets a value indicating whether the device is connected or not.
        /// </summary>
        /// <value>Connection state.</value>
        public bool IsConnected
        {
            get
            {
                return this.isConnected;
            }
        }

        /// <summary>
        /// Tries to connect to a device.
        /// </summary>
        /// <returns>Connection state.</returns>
        public bool Connect()
        {
            this.isConnected = false;

            // Variables we will use when loading this form
            int devNum = 0;
            StringBuilder devStr = new StringBuilder(SLUSBXpressDLL.SI_MAX_DEVICE_STRLEN);
            int i;

            SLUSBXpressDLL.Status = SLUSBXpressDLL.SI_GetNumDevices(ref devNum);

            // if we find a device, obtain the VID and PID of each device
            if (SLUSBXpressDLL.Status == SLUSBXpressDLL.SI_SUCCESS)
            {
                for (i = 0; i < devNum; i++)
                {
                    SLUSBXpressDLL.Status = SLUSBXpressDLL.SI_GetProductString(i, devStr, SLUSBXpressDLL.SI_RETURN_VID);
                    if (devStr.ToString().ToUpper() == this.vid.ToString("X4"))
                    {
                        SLUSBXpressDLL.Status = SLUSBXpressDLL.SI_GetProductString(i, devStr, SLUSBXpressDLL.SI_RETURN_PID);
                        if (devStr.ToString().ToUpper() == this.pid.ToString("X4"))
                        {
                            // and open the device
                            if (SLUSBXpressDLL.SI_SUCCESS == SLUSBXpressDLL.SI_Open(i, ref SLUSBXpressDLL.HUSBDevice))
                            {
                                if (SLUSBXpressDLL.SI_SUCCESS == SLUSBXpressDLL.SI_SetBaudRate(SLUSBXpressDLL.HUSBDevice, this.baudrate))
                                {
                                    // Sets the lineparameter, Datenbits(8) * 256 + Parity(2) * 8 + Stopbit(0)
                                    int lineParameter = (this.databits * 256) + (this.parity * 16) + this.stopbit;
                                    if (SLUSBXpressDLL.SI_SUCCESS == SLUSBXpressDLL.SI_SetLineControl(SLUSBXpressDLL.HUSBDevice, lineParameter))
                                    {
                                        if (SLUSBXpressDLL.SI_SUCCESS == SLUSBXpressDLL.SI_SetFlowControl(SLUSBXpressDLL.HUSBDevice, SLUSBXpressDLL.SI_STATUS_INPUT, SLUSBXpressDLL.SI_FIRMWARE_CONTROLLED, SLUSBXpressDLL.SI_FIRMWARE_CONTROLLED, SLUSBXpressDLL.SI_STATUS_INPUT, SLUSBXpressDLL.SI_STATUS_INPUT, 0))
                                        {
                                            if (SLUSBXpressDLL.SI_SUCCESS == SLUSBXpressDLL.SI_SetTimeouts(this.rtimeout, this.wtimeout))
                                            {
                                                this.isConnected = true;
                                                break;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            return this.isConnected;
        }

        /// <summary>
        /// Disconnect from the device.
        /// </summary>
        public void Disconnect()
        {
            SLUSBXpressDLL.Status = SLUSBXpressDLL.SI_Close(SLUSBXpressDLL.HUSBDevice);

            if (SLUSBXpressDLL.Status == SLUSBXpressDLL.SI_SUCCESS)
            {
                this.isConnected = false;
            }
        }

        /// <summary>
        /// Send a command to the device
        /// Pattern of the sent data:
        /// 1 byte: STX
        /// 1 byte: package number (always NULL)
        /// 2 byte: data length (Big-Endian)
        /// n byte: data
        /// 2 byte: CRC16-checksum (data bytes may not be encoded).
        /// </summary>
        /// <param name="opcode">Opcode of the command.</param>
        /// <param name="parameter">List of parameters.</param>
        /// <param name="readLength">Indicates how many bytes should be read as answer after sending the command.</param>
        /// <returns>Read answer from DDS130.</returns>
        public byte[] SendToDevice(byte[] data, int readLength)
        {
            if (this.Write(data))
            {
                Thread.Sleep(500);
                return this.Read(readLength);
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// Writes or rather sends a command to the device.
        /// </summary>
        /// <param name="command">Command as byte-array to process.</param>
        /// <returns>Result of the action.</returns>
        private bool Write(byte[] command)
        {
            int bytesSucceed = 0;
            int bytesWriteRequest = command.Length;

            SLUSBXpressDLL.Status = SLUSBXpressDLL.SI_FlushBuffers(SLUSBXpressDLL.HUSBDevice, 1, 1);
            SLUSBXpressDLL.Status = SLUSBXpressDLL.SI_Write(SLUSBXpressDLL.HUSBDevice, ref command[0], bytesWriteRequest, ref bytesSucceed, 0);

            if ((bytesSucceed != bytesWriteRequest) || (SLUSBXpressDLL.Status != SLUSBXpressDLL.SI_SUCCESS))
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        /// <summary>
        /// Reads data from the device.
        /// </summary>
        /// <param name="bufSize">Input buffer size.</param>
        /// <returns>Data that was readed.</returns>
        private byte[] Read(int bufSize)
        {
            byte[] buffer = new byte[bufSize + 6];
            int bytesSucceed = 0;
            int bytesReadRequest = bufSize;

            SLUSBXpressDLL.Status = SLUSBXpressDLL.SI_Read(SLUSBXpressDLL.HUSBDevice, ref buffer[0], bytesReadRequest, ref bytesSucceed, 0);

            if (SLUSBXpressDLL.Status != SLUSBXpressDLL.SI_SUCCESS)
            {
                return null;
            }

            if (buffer[0] != SiUSBXpDevice.STX)
            {
                Console.WriteLine(ASCIIEncoding.Default.GetString(buffer));
                return null;
            }

            return buffer;
        }
    }
}