﻿//-----------------------------------------------------------------------------
// <copyright file="BitUtils.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Utility class for providing methods for byte manipulation.</summary>
//-----------------------------------------------------------------------------
namespace DDSCommunication.Utils
{
    using System;
    using System.Collections;
    using System.Text;

    /// <summary>
    /// Utility class for providing methods for byte manipulation.
    /// </summary>
    public static class BitUtils
    {
        /// <summary>
        /// Gets a specific bit of a byte.
        /// </summary>
        /// <param name="b">Used byte.</param>
        /// <param name="index">Index of the wanted bit.</param>
        /// <returns>State of the bit.</returns>
        public static bool GetBit(byte b, int index)
        {
            byte add = (byte)Math.Pow(2, index);
            return (b & add) != 0;
        }

        /// <summary>
        /// Sets a specific bit of a byte.
        /// </summary>
        /// <param name="b">Used byte.</param>
        /// <param name="index">Index of the wanted bit.</param>
        /// <param name="state">Wished state of the wanted bit.</param>
        public static void SetBit(ref byte b, int index, bool state)
        {
            byte add = (byte)Math.Pow(2, index);
            if ((b & add) != 0)
            {
                if (!state)
                {
                    b -= add;
                }
            }
            else
            {
                if (state)
                {
                    b += add;
                }
            }
        }

        /// <summary>
        /// Converts a BitArray to a string representation (f.e. "1001100").
        /// </summary>
        /// <param name="bitArray">BitArray to convert.</param>
        /// <returns>Converted BitArray.</returns>
        public static string ConvertBitArrayToString(BitArray bitArray)
        {
            if (bitArray == null)
            {
                return string.Empty;
            }

            StringBuilder sb = new StringBuilder();
            foreach (var bit in bitArray)
            {
                sb.Append((bool)bit ? "1" : "0");
            }

            return sb.ToString();
        }
    }
}